Index: dia/dia-0.86/app/render_libart.c
diff -u dia/dia-0.86/app/render_libart.c:1.1.1.1 dia/dia-0.86/app/render_libart.c:1.2
--- dia/dia-0.86/app/render_libart.c:1.1.1.1	Tue Aug  8 15:12:49 2000
+++ dia/dia-0.86/app/render_libart.c	Sat Aug 26 14:18:32 2000
@@ -1107,6 +1107,92 @@
   art_svp_free( svp );
 }
 
+#ifdef USE_XIM
+SuckFont *
+get_suck_font (GdkFont *font, guchar *sentence)
+{
+  SuckFont *suckfont;
+  int x, y;
+  char text[2];
+  int lbearing, rbearing, ch_width, ascent, descent;
+  GdkPixmap *pixmap;
+  GdkColor black, white;
+  GdkImage *image;
+  GdkGC *gc;
+  guchar *line;
+  int width, height;
+  int black_pixel, pixel;
+
+  if (!font)
+    return NULL;
+
+  suckfont = g_new (SuckFont, 1);
+
+  height = font->ascent + font->descent;
+  x = 0;
+  gdk_string_extents(font, sentence,
+		     &lbearing, &rbearing, &ch_width, &ascent, &descent);
+  suckfont->chars[0].left_sb = lbearing;
+  suckfont->chars[0].right_sb = ch_width - rbearing;
+  suckfont->chars[0].width = rbearing - lbearing;
+  suckfont->chars[0].ascent = ascent;
+  suckfont->chars[0].descent = descent;
+  suckfont->chars[0].bitmap_offset = x;
+  x += (ch_width + 31) & -32;
+  width = x;
+
+  suckfont->bitmap_width = width;
+  suckfont->bitmap_height = height+1;
+  suckfont->ascent = font->ascent+1;
+
+  pixmap = gdk_pixmap_new (NULL, suckfont->bitmap_width,
+			   suckfont->bitmap_height, 1);
+  gc = gdk_gc_new (pixmap);
+  gdk_gc_set_font (gc, font);
+
+  black_pixel = color_gdk_black.pixel;
+  black.pixel = black_pixel;
+  white.pixel = color_gdk_white.pixel;
+  gdk_gc_set_foreground (gc, &white);
+  gdk_draw_rectangle (pixmap, gc, 1, 0, 0, suckfont->bitmap_width, suckfont->bitmap_height);
+
+  gdk_gc_set_foreground (gc, &black);
+
+  gdk_draw_string (pixmap, font, gc,
+		   suckfont->chars[0].bitmap_offset - suckfont->chars[0].left_sb,
+		   font->ascent+1,
+		   sentence);
+
+  /* The handling of the image leaves me with distinct unease.  But this
+   * is more or less copied out of gimp/app/text_tool.c, so it _ought_ to
+   * work. -RLL
+   */
+
+  image = gdk_image_get (pixmap, 0, 0, suckfont->bitmap_width, suckfont->bitmap_height);
+  suckfont->bitmap = g_malloc0 ((width >> 3) * suckfont->bitmap_height);
+
+  line = suckfont->bitmap;
+  for (y = 0; y < suckfont->bitmap_height; y++) {
+    for (x = 0; x < suckfont->bitmap_width; x++) {
+      pixel = gdk_image_get_pixel (image, x, y);
+      if (pixel == black_pixel)
+	line[x >> 3] |= 128 >> (x & 7);
+    }
+    line += width >> 3;
+  }
+
+  gdk_image_destroy (image);
+
+  /* free the pixmap */
+  gdk_pixmap_unref (pixmap);
+
+  /* free the gc */
+  gdk_gc_destroy (gc);
+
+  return suckfont;
+}
+#endif
+
 static void
 draw_string(RendererLibart *renderer,
 	    const char *text,
@@ -1121,6 +1207,9 @@
   SuckFont *suckfont;
   double xpos, ypos;
   guint32 rgba;
+#ifdef USE_XIM
+  SuckChar *ch;
+#endif
 
   ddisplay_transform_coords(ddisp, pos->x, pos->y,
 			    &x, &y);
@@ -1137,7 +1226,11 @@
     break;
   }
 
+#ifdef USE_XIM
+  suckfont = get_suck_font(renderer->gdk_font, text);
+#else
   suckfont = renderer->suck_font;
+#endif
 
   xpos = (double) x + 1; 
   ypos = (double) (y - suckfont->ascent);
@@ -1145,6 +1238,22 @@
   rgba = color_to_rgba(color);
   
   len = strlen(text);
+#ifdef USE_XIM
+  ch = &suckfont->chars[0];
+  art_affine_translate(affine, xpos, ypos);
+  art_rgb_bitmap_affine(renderer->rgb_buffer,
+			0, 0,
+			renderer->renderer.pixel_width,
+			renderer->renderer.pixel_height,
+			renderer->renderer.pixel_width*3,
+			suckfont->bitmap + (ch->bitmap_offset >> 3),
+			ch->width,
+			suckfont->bitmap_height,
+			suckfont->bitmap_width >> 3,
+			rgba,
+			affine,
+			ART_FILTER_NEAREST, NULL);
+#else
   for (i = 0; i < len; i++) {
     SuckChar *ch;
     
@@ -1166,7 +1275,7 @@
     dx = ch->left_sb + ch->width + ch->right_sb;
     xpos += dx;
   }
-
+#endif
   
   
 }
